# OB - the object library !
#
#

import datetime, json, os, random, uuid

from .utl import cdir, get_type, hooked, xdir

def __dir__():
    return ("Object", "ObjectDecoder", "ObjectEncoder", "Ol", "edit", "find", "format", "get", "items", "keys", "load", "register", "save", "search", "stamp", "update", "values", "workdir")

workdir = ""

class Object:

    __slots__ = ("__dict__", "__stamp__")

    def __init__(self):
        self.__stamp__ = os.path.join(get_type(self), str(uuid.uuid4()), str(datetime.datetime.now()).replace(" ", os.sep))

    def __delitem__(self, k):
        del self.__dict__[k]

    def __getitem__(self, k, d=None):
        return self.__dict__.get(k, d)

    def __iter__(self):
        return iter(self.__dict__.keys())

    def __len__(self):
        return len(self.__dict__)

    def __lt__(self, o):
        return len(self) < len(o)

    def __setitem__(self, k, v):
        self.__dict__[k] = v
        return self.__dict__[k]

    def __str__(self):
        return json.dumps(self, cls=ObjectEncoder, indent=4, sort_keys=True)

class ObjectEncoder(json.JSONEncoder):

    def default(self, o):
        if isinstance(o, Object):
            return vars(o)
        if isinstance(o, dict):
            return o.items()
        if isinstance(o, list):
            return iter(o)
        if isinstance(o, (type(str), type(True), type(False), type(int), type(float))):
            return o
        return repr(o)

class ObjectDecoder(json.JSONDecoder):

    def decode(self, o):
        return json.loads(o, object_hook=hooked)

class Ol(Object):

    def append(self, key, value):
        if key not in self:
            self[key] = []
        if isinstance(value, type(list)):
            self[key].extend(value)
        else:
            if value not in self[key]:
                self[key].append(value)

    def update(self, d):
        for k, v in d.items():
            self.append(k, v)

def edit(o, setter, skip=False):
    try:
        setter = vars(setter)
    except (TypeError, ValueError):
        pass
    if not setter:
        setter = {}
    count = 0
    for key, value in setter.items():
        if skip and value == "":
            continue
        count += 1
        if value in ["True", "true"]:
            o[key] = True
        elif value in ["False", "false"]:
            o[key] = False
        else:
            o[key] = value
    return count

def find(o, txt):
    for k, v in items(o):
        if txt in str(v):
            return True
    return False

def format(o, keys=None, pure=False, skip=None):
    if not keys:
        keys = vars(o).keys()
    res = []
    txt = ""
    for key in keys:
        if skip and key in skip:
            continue
        if key == "stamp":
            continue
        try:
            val = o[key]
        except KeyError:
            continue
        if not val:
            continue
        val = str(val)
        if key == "text":
            val = val.replace("\\n", "\n")
        res.append((key, val))
    for k, v in res:
        if pure:
            txt += "%s%s" % (str(v).strip(), " ")
        else:
            txt += "%s=%s%s" % (k, str(v).strip(), " ")
    return txt.strip()

def get(o, k, d=None):
    try:
        return o.get(k, d)
    except (TypeError, AttributeError):
        return o.__dict__.get(k, d)

def items(o):
    try:
        return o.items()
    except (TypeError, AttributeError):
        return o.__dict__.items()

def keys(o):
    try:
        return o.keys()
    except (TypeError, AttributeError):
        return o.__dict__.keys()

def load(o, path, force=False):
    assert path
    assert workdir
    o.__stamp__ = path
    lpath = os.path.join(workdir, "store", path)
    cdir(lpath)
    with open(lpath, "r") as ofile:
        try:
            v = json.load(ofile, cls=ObjectDecoder)
        except json.decoder.JSONDecodeError as ex:
            print(path, ex)
            return
        if v:
            if isinstance(v, Object):
                o.__dict__.update(vars(v))
            else:
                o.__dict__.update(v)

def register(o, k, v):
    o[k] = v

def save(o, stime=None):
    assert workdir
    if stime:
        o.__stamp__ = os.path.join(get_type(o), str(uuid.uuid4()), stime + "." + str(random.randint(0,100000)))
    else:
        o.__stamp__ = os.path.join(get_type(o), str(uuid.uuid4()), str(datetime.datetime.now()).replace(" ", os.sep))
    opath = os.path.join(workdir, "store", o.__stamp__)
    cdir(opath)
    with open(opath, "w") as ofile:
        json.dump(stamp(o), ofile, cls=ObjectEncoder)
    os.chmod(opath, 0o444)
    return o.__stamp__

def search(o, s):
    ok = False
    for k, v in items(s):
        vv = get(o, k)
        if v not in str(vv):
            ok = False
            break
        ok = True
    return ok

def stamp(o):
    for k in xdir(o):
        oo = getattr(o, k, None)
        if isinstance(oo, Object):
            stamp(oo)
            oo.__dict__["stamp"] = oo.__stamp__
            o[k] = oo
        else:
            continue
    o.__dict__["stamp"] = o.__stamp__
    return o

def update(o, d):
    if isinstance(d, Object):
        return o.__dict__.update(vars(d))
    return o.__dict__.update(d)

def values(o):
    try:
        return o.values()
    except (TypeError, AttributeError):
        return o.__dict__.values()
