# OB - the object library !
#
#

import ob, os

from .tms import fntime
from .utl import get_type, hook

class Db(ob.Object):

    def all(self, otype, selector=None, index=None, timed=None):
        nr = -1
        if selector is None:
            selector = {}
        for fn in names(otype, timed):
            o = hook(fn)
            if selector and not ob.search(o, selector):
                continue
            if "_deleted" in o and o._deleted:
                continue
            nr += 1
            if index is not None and nr != index:
                continue
            yield o

    def deleted(self, otype):
        for fn in names(otype):
            o = hook(fn)
            if "_deleted" not in o or not o._deleted:
                continue
            yield o

    def find(self, otype, selector=None, index=None, timed=None):
        nr = -1
        if selector is None:
            selector = {}
        for fn in names(otype, timed):
            o = hook(fn)
            if selector and not ob.search(o, selector):
                continue
            if "_deleted" in o and o._deleted:
                continue
            nr += 1
            if index is not None and nr != index:
                continue
            yield o

    def find_event(self, e):
        nr = -1
        for fn in names(e.otype, e.timed):
            o = hook(fn)
            if e.gets and not ob.search(o, e.gets):
                continue
            if "_deleted" in o and o._deleted:
                continue
            nr += 1
            if e.index is not None and nr != e.index:
                continue
            yield o

    def find_value(self, otype, value, index=None, timed=None):
        nr = -1
        for fn in names(otype, timed):
            o = hook(fn)
            if o.find(value):
                nr += 1
                if index is not None and nr != index:
                    continue
                if "_deleted" in o and o._deleted:
                    continue
                yield o

    def last(self, otype):
        fns = names(otype)
        if fns:
            fn = fns[-1]
            return hook(fn)

    def last_fn(self, otype):
        fns = names(otype)
        if fns:
            fn = fns[-1]
            return (fn, hook(fn))
        return (None, None)

    def last_all(self, otype, selector=None, index=0, timed=None):
        nr = -1
        res = []
        if not selector:
            selector = {}
        for fn in names(otype, timed):
            o = hook(fn)
            if selector is not None and ob.search(o, selector):
                nr += 1
                if index is not None and nr != index:
                    continue
                res.append((fn, o))
            else:
                res.append((fn, o))
        if res:
            s = sorted(res, key=lambda x: fntime(x[0]))
            if s:
                return s[-1][-1]
        return None

def last(o):
    db = Db()
    path, l = db.last_fn(str(get_type(o)))
    if  l:
        ob.update(o, l)
        o.__stamp__ = path

def names(name, timed=None):
    if not name:
        return []
    assert ob.workdir
    p = os.path.join(ob.workdir, "store", name) + os.sep
    res = []
    for rootdir, dirs, files in os.walk(p, topdown=False):
        for fn in files:
            fnn = os.path.join(rootdir, fn).split(os.path.join(ob.workdir, "store"))[-1]
            ftime = fntime(fnn)
            if timed and "from" in timed and timed["from"] and ftime < timed["from"]:
                continue
            if timed and timed.to and ftime > timed.to:
                continue
            res.append(os.sep.join(fnn.split(os.sep)[1:]))
    return sorted(res, key=fntime)
